{--------------------------------------------------------}
{
{    VoxTube - utilitrio de acessibilizao do YouTube
{
{    Autores:  Antonio Borges e Fabiano Ferreira
{
{    Em fevereiro de 2013
{
{--------------------------------------------------------}

program voxtube;

uses
  dvcrt,
  dvwin,
  dvForm,
  dvinet,
  dvexec,
  vtmsg,
  SysUtils,
  superObject,
  windows,
  shellapi;

const
    DEBUG = false;
    VERSAO = '1.2';

var
    sock: integer;
    lido: ISuperObject;
    listaDeFilmes: TSuperArray;
    iniciandoProg: boolean;

var
    titulo, autor, conteudo: string;
    duracao, visualizacoes: integer;
    paginaWeb: string;
    quantNoYouTube: integer;

{--------------------------------------------------------}
{   transforma string para a codificao usada em URLs
{--------------------------------------------------------}

function stringToURL(s: string): string;
var i: integer;
begin
    result := '';
    for i := 1 to length(s) do
        begin
            if s[i] in ['0'..'9', 'a'..'z', 'A'..'Z', '.', '-', '_', '~'] then
                result := result + s[i]
            else
                result := result + '%' + intToHex(ord(s[i]), 2);
        end;
end;

{--------------------------------------------------------}
{           faz o pedido de busca ao youtube
{--------------------------------------------------------}

function trazDoYoutube (busca: string; inicio: integer): string;
var
    pbuf: PbufRede;
    s: string;
    c: char;
    ok: boolean;
begin
    abreWinsock;
    sock := abreConexao ('gdata.youtube.com', 80);
    if sock <= 0 then
        begin
            mensagem('VTNAOCON', 1); {'No consegui realizar a conexo.'}
            result := '';
            exit;
        end;

    busca := stringToURL(busca);

    writelnRede (sock, 'GET /feeds/api/videos?q=' + busca +
                       '&start-index=' + intToStr(inicio) +
                       '&alt=json' +
                       ' HTTP/1.0');
    writelnRede (sock, 'Host: gdata.youtube.com');
    writelnRede (sock, 'User-Agent: Webvox 5.2d');
    writelnRede (sock, 'Accept-Language: pt-br');
    writelnRede (sock, 'UA-CPU: x86');
    writelnRede (sock, '');

    pbuf := inicBufRede (sock);
    repeat
        ok := readlnBufRede(pbuf, s, 5000);
    until (not ok) or (s = '');

    s := '';
    repeat
        while not temDadoBufRede(pbuf) do delay (1000);
        ok := leCaracBufRede(pbuf, c);
        if ok then s := s + c;
    until not ok;

    fimBufRede (pbuf);
    fechaConexao (sock);
    result := s;
    fechaWinsock;
end;

{--------------------------------------------------------}
{           remove caracteres de formatao html
{--------------------------------------------------------}

function formata (s: string): string;
var
    saida: string;
begin
    saida := '';
    textColor (White);
    while (s <> '') do
        if copy (s, 1, 6) = '&quot;' then
            begin
                saida := saida + '"';
                delete (s, 1, 6);
            end
        else
        if copy (s, 1, 5) = '&#39;' then
            begin
                saida := saida + '''';
                delete (s, 1, 5);
            end
        else
        if (copy (s, 1, 3) = '<b>') OR
           (copy (s, 1, 3) = '<i>') then   delete (s, 1, 3)
        else
        if (copy (s, 1, 4) = '</b>') or
           (copy (s, 1, 4) = '</i>') then  delete (s, 1, 4)

        else
            begin
                saida := saida + s[1];
                delete (s, 1, 1);
            end;
    result := saida;
end;

{--------------------------------------------------------}
{ rotina auxiliar para guardar o formato json para debug
{--------------------------------------------------------}

procedure debuga (s: string);
var
    arq: textfile;
begin
    if not DEBUG then exit;
    assignFile (arq, 'debug.json');
    rewrite (arq);
    write (arq, s);
    closefile (arq);
end;

{--------------------------------------------------------}
{         monta a estrutura para folhear os filmes
{--------------------------------------------------------}

function montaFolheamento (s: string; indice_atual: integer): boolean;   // retorna tambm a lista de filmes
var
    filme: ISuperObject;
    titulo: string;
    wstring: WideString;
    i: integer;
    nLidos: integer;
begin
    wstring := s;  // Superobject s trabalha com widestring
    lido := TSuperObject.ParseString2(pWideChar(wstring));

    try
        listaDeFilmes := lido['feed']['entry'].AsArray;
        nlidos := listaDeFilmes.Length;
        quantNoYouTube := lido['feed']['openSearch$totalResults']['$t'].asInteger;
    except
        quantNoYouTube := 0;
        nlidos := 0;
    end;

    window (1, 1, 80, 25);
    gotoxy (1, 3);
    clreol;
    mensagem('VTNUMLID', 0);  {'Nmero de registros lidos: '}
    sintWriteInt (nlidos);
    if (quantNoYouTube <> 0) and (indice_atual = 1) then
        begin
            mensagem('VTUNIVER', 0);   {' de um universo de '}
            sintWriteInt (quantNoYouTube);
        end;
    writeln;
    textbackground (RED);
    writeln ('-------------------------------------------------------------------------------');
    textbackground (BLACK);

    window (1, wherey, 80, 25);
    folheiaCria (wherex, wherey, 80, 20);
    for i := 0 to nlidos-1 do
        begin
            filme := listaDeFilmes.O[i];
            s := filme['title']['$t'].AsString;
            titulo := formata (Utf8ToAnsi(s));
            folheiaAdiciona(titulo);
        end;

    result := nlidos <> 0;
end;

{--------------------------------------------------------}
{            Descobre qual  o navegador padro
{--------------------------------------------------------}

function GetDefaultBrowser: string;
var
    tmp : PChar;
    res : PChar;
begin
    tmp := StrAlloc(255);
    res := StrAlloc(255);
    try
        GetTempPath(255,tmp);
        FileCreate(tmp+'htmpl.htm');
        FindExecutable('htmpl.htm',tmp,Res);
        Result := ExtractFilePath(res) + ExtractFileName(res);
        SysUtils.DeleteFile(tmp+'htmpl.htm');
    finally
        StrDispose(tmp);
        StrDispose(res);
    end;
end;

{--------------------------------------------------------}
{       mostra o youtube no navegador web padro
{--------------------------------------------------------}

procedure mostraWeb (pagina: string);
begin
    mensagem ('VTABNAV', 1);  {'Abrindo navegador. Acione ALT F4 quando terminar.'}
    while sintFalando do waitMessage;
    delay (100);

    executaProg(GetDefaultBrowser, '', pagina);
    delay (5000);
    while getForegroundWindow <> crtWindow do delay (500);
end;

{--------------------------------------------------------}
{          obtem as informaes do filme
{--------------------------------------------------------}

procedure obtemInfoFilme (ultFolheado: integer);
var
    s: string;
    autores: TSuperArray;
    filmeSelec: ISuperObject;
    links: TSuperArray;

begin
    filmeSelec := listaDeFilmes.O[ultFolheado-1];
    s := filmeSelec['title']['$t'].AsString;

    titulo := formata (Utf8ToAnsi(s));
    s := filmeSelec['content']['$t'].AsString;
    conteudo := formata (Utf8ToAnsi(s));

    autores := filmeSelec['author'].AsArray;
    s := autores[0]['name']['$t'].AsString;
    autor := formata (Utf8ToAnsi(s));

    links := filmeSelec['link'].AsArray;
    paginaWeb := links[0]['href'].AsString;

    try    duracao := filmeSelec['media$group']['yt$duration']['seconds'].asInteger;
    except duracao := 0;
    end;

    try    visualizacoes := filmeSelec['yt$statistics']['viewCount'].asInteger;
    except visualizacoes := 0;
    end;
end;

{--------------------------------------------------------}

procedure ajuda;
begin
    writeln;
    mensagem ('VTOPCAO', 1);   {'As opes so:'}
    mensagem ('VTOP_ENT', 1);  {'ENTER - inicia a execuo do vdeo'}
    mensagem ('VTOP_I', 1);    {'i - informaes sobre o vdeo'}
    mensagem ('VTOP_P', 1);    {'p - prximos 25 vdeos (ou quantos houver)'}
    mensagem ('VTOP_A', 1);    {'a - vdeos anteriores'}
    mensagem ('VTOP_Q', 1);    {'q informa a posio na lista de vdeos'}
    mensagem ('VTOP_CTC', 1);  {'ctrl+c - copia o ttulo e o link dos vdeos'}
    mensagem ('VTOP_CTL', 1);  {'ctrl+l - copia o link do vdeo atual'}
    mensagem ('VTOP_ESC', 2);  {'ESC - Cancelar'}
    readkey;
    limpaBufTec;
end;

{--------------------------------------------------------}

procedure menuAdiciona (cod: string);
begin
    popupMenuAdiciona (cod, pegaTextoMensagem(cod));
end;

function selSetasOpcao: char;
var n: integer;
const
    opmenu: string = ^m + 'IPAQ' + ^C + ^L + #$1b;
begin
    mensagem ('VTSELSET', 2); {'Selecione com as setas a opo desejada:'}

    popupMenuCria(wherex, wherey, 50, 8, RED);
    MenuAdiciona ('VTOP_ENT');  {'ENTER - inicia a execuo do vdeo'}
    MenuAdiciona ('VTOP_I');    {'i - informaes sobre o vdeo'}
    MenuAdiciona ('VTOP_P');    {'p - prximos 25 vdeos (ou quantos houver)'}
    MenuAdiciona ('VTOP_A');    {'a - vdeos anteriores'}
    MenuAdiciona ('VTOP_Q');    {'q informa a posio na lista de vdeos'}
    MenuAdiciona ('VTOP_CTC');  {'ctrl+c - copia o ttulo e o link dos vdeos'}
    MenuAdiciona ('VTOP_CTL');  {'ctrl+l - copia o link do vdeo atual'}
    MenuAdiciona ('VTOP_ESC');  {'ESC - Cancelar'}

    n := popupMenuSeleciona;
    if (n < 1) then
        result := ' '
    else
        result := opmenu[n];
end;

{--------------------------------------------------------}
{       folheia os vdeos informados pelo youtube
{--------------------------------------------------------}

procedure processaFuncao (c1, c2: char;
                         ultFolheado: integer;
                         var indice_atual: integer;
                         var mudouPagina: boolean);
var clipboardAcum : string;
    i: integer;
    texto: string;
    estaSelec: boolean;
const
    CRLF = ^m^j;
label denovo;

begin
denovo:
    clrscr;

    case upcase(c1) of
        ESC:  begin
                  indice_atual := 0;
                  mensagem ('VTFOLTRM', 1); {'Folheamento terminado'}
              end;

        ENTER:
            begin
                obtemInfoFilme (ultFolheado);
                mostraWeb (paginaWeb);
            end;

        'Q': sintetiza (intToStr (indice_atual+ultFolheado-1));

        'P': begin
                indice_atual := indice_atual + 25;
                if indice_atual > quantNoYouTube then
                    begin
                       sintBip;
                       indice_atual := indice_atual - 25;
                    end;
                mudouPagina := true;
             end;

        'A': begin
                 if indice_atual > 25 then
                     indice_atual := indice_atual - 25
                 else
                     begin
                         if indice_atual = 1 then sintBip;
                         indice_atual := 1;
                     end;
                mudouPagina := true;
            end;

        'I': begin
                 obtemInfoFilme (ultFolheado);

                 writeln (titulo);
                 sintClek;
                 mensagem ('VTAUTOR', 0);  {'Autor: '}
                 textColor (yellow);
                 sintWriteln (Autor);

                 sintClek;
                 textColor (white);
                 mensagem ('VTDURA', 0);   {'Durao: '}
                 textColor (yellow);
                 if (duracao div 60) = 0 then
                     sintWriteln (intToStr (duracao mod 60) + pegaTextoMensagem ('VTSEGUND'))  {' segundos'}
                 else
                     sintWriteln (intToStr (duracao div 60) + pegaTextoMensagem ('VTMINUTO')    {' minutos '}
                                + intToStr (duracao mod 60) + pegaTextoMensagem ('VTSEGUND'));  {' segundos'}

                 sintClek;
                 textColor (white);
                 mensagem ('VTVISUAL', 0);  {'Visualizaes: '}
                 textColor (yellow);
                 sintWriteln (intToStr(visualizacoes));

                 sintClek;
                 textColor (yellow);
                 sintWriteln (conteudo);
                 textColor (white);
                 writeln;

                 mensagem ('VTAPTENT', 0);    {'Aperte enter...'}
                 readln;
             end;

        ^c:  // control-c
             begin
                clipboardAcum := '';
                for i := 1 to folheiaNumItens do
                    begin
                        folheiaObtemItem(i, texto, estaSelec);
                        if estaSelec then
                            begin
                                obtemInfoFilme(i);
                                clipBoardAcum := clipboardAcum + titulo + CRLF +
                                                                 paginaweb + CRLF;
                            end;
                    end;
                if clipboardAcum = '' then
                    begin
                        obtemInfoFilme(ultFolheado);
                        clipboardAcum := titulo + crlf + paginaWeb + crlf;
                    end;
                putClipboard (pchar(clipboardAcum));
                sintclek;
            end;

        ^l:
            begin
                obtemInfoFilme(ultFolheado);
                clipboardAcum := paginaWeb + crlf;
                putClipboard (pchar(clipboardAcum));
                sintclek;
            end;

        #0:  case c2 of
                 F1: ajuda;
                 F9: begin
                        if ultFolheado > 0 then
                            begin
                                obtemInfoFilme (ultFolheado);
                                writeln (titulo);
                                writeln;
                            end;

                         c1 := selSetasOpcao;
                         goto denovo;
                     end;
             end;
    end;
end;

{--------------------------------------------------------}
{       folheia os vdeos informados pelo youtube
{--------------------------------------------------------}

procedure folheiaVideos (var indice_atual: integer);
var c1, c2: char;
    ultFolheado: integer;
    mudouPagina: boolean;
const
    DESCONHECIDO = 'Desconhecido';

begin
    ultFolheado := 1;
    repeat
        clrscr;

        if folheiaExecuta(ultFolheado, ultFolheado, c1, c2, true) then
            processaFuncao (c1, c2, ultFolheado, indice_atual, mudouPagina)
        else
            sintBip;

    until mudouPagina or (indice_atual = 0);
end;

{--------------------------------------------------------}
{                   mostra ou fala cabealho
{--------------------------------------------------------}

procedure cabecalho;
begin
    textbackground (BLUE);
    if iniciandoProg then
        begin
            mensagem ('VTINIC',0);    {'Programa de Pesquisa no YouTube'}
            sintWriteln (VERSAO);
            iniciandoProg := false;
        end
    else
        writeln (pegaTextoMensagem('VTINIC') + VERSAO);
    textbackground (BLACK);
end;

{--------------------------------------------------------}
{                   programa principal
{--------------------------------------------------------}

procedure processaBusca (busca: string);
var
    indice_atual: integer;
    s: string;
begin
    indice_atual := 1;
    repeat
        gotoxy (1, 3);
        s := trazDoYoutube (busca, indice_atual);
        debuga (s);
        if s <> '' then
            begin
                if montaFolheamento (s, indice_atual) then
                     folheiaVideos (indice_atual);
                folheiaDestroi;
            end;
    until (s = '') or (quantNoYouTube = 0) or (indice_atual = 0);
end;

{--------------------------------------------------------}
{                   programa principal
{--------------------------------------------------------}

var
    c: char;
    busca: string;

label inicio;

begin
    clrscr;
    setWindowTitle('VoxTube ' + VERSAO);
    iniciandoProg := true;
    sintInic (0, '');

    repeat
        window (1, 1, 80, 25);
        clrscr;
        cabecalho;

        mensagem ('VTQBUSCA',0);  {'Qual sua busca: '}
        sintReadln (busca);

        processaBusca (busca);

        clrscr;
        mensagem ('VTOUTRO', 0);   {'Deseja pesquisar outro assunto? '}
        c := sintReadkey;
        writeln (c);
    until upcase(c) <> 'S';

    clrscr;
    mensagem ('VTFIM',1); {'Fim do programa'}
    delay (1000);
    sintfim;
    doneWinCrt;
end.
